# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: earth_osm/osmpbf/osmformat.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n earth_osm/osmpbf/osmformat.proto\x12\x06OSMPBF\"\x87\x02\n\x0bHeaderBlock\x12 \n\x04\x62\x62ox\x18\x01 \x01(\x0b\x32\x12.OSMPBF.HeaderBBox\x12\x19\n\x11required_features\x18\x04 \x03(\t\x12\x19\n\x11optional_features\x18\x05 \x03(\t\x12\x16\n\x0ewritingprogram\x18\x10 \x01(\t\x12\x0e\n\x06source\x18\x11 \x01(\t\x12%\n\x1dosmosis_replication_timestamp\x18  \x01(\x03\x12+\n#osmosis_replication_sequence_number\x18! \x01(\x03\x12$\n\x1cosmosis_replication_base_url\x18\" \x01(\t\"F\n\nHeaderBBox\x12\x0c\n\x04left\x18\x01 \x02(\x12\x12\r\n\x05right\x18\x02 \x02(\x12\x12\x0b\n\x03top\x18\x03 \x02(\x12\x12\x0e\n\x06\x62ottom\x18\x04 \x02(\x12\"\xd2\x01\n\x0ePrimitiveBlock\x12(\n\x0bstringtable\x18\x01 \x02(\x0b\x32\x13.OSMPBF.StringTable\x12.\n\x0eprimitivegroup\x18\x02 \x03(\x0b\x32\x16.OSMPBF.PrimitiveGroup\x12\x18\n\x0bgranularity\x18\x11 \x01(\x05:\x03\x31\x30\x30\x12\x15\n\nlat_offset\x18\x13 \x01(\x03:\x01\x30\x12\x15\n\nlon_offset\x18\x14 \x01(\x03:\x01\x30\x12\x1e\n\x10\x64\x61te_granularity\x18\x12 \x01(\x05:\x04\x31\x30\x30\x30\"\xb7\x01\n\x0ePrimitiveGroup\x12\x1b\n\x05nodes\x18\x01 \x03(\x0b\x32\x0c.OSMPBF.Node\x12!\n\x05\x64\x65nse\x18\x02 \x01(\x0b\x32\x12.OSMPBF.DenseNodes\x12\x19\n\x04ways\x18\x03 \x03(\x0b\x32\x0b.OSMPBF.Way\x12#\n\trelations\x18\x04 \x03(\x0b\x32\x10.OSMPBF.Relation\x12%\n\nchangesets\x18\x05 \x03(\x0b\x32\x11.OSMPBF.ChangeSet\"\x18\n\x0bStringTable\x12\t\n\x01s\x18\x01 \x03(\x0c\"q\n\x04Info\x12\x13\n\x07version\x18\x01 \x01(\x05:\x02-1\x12\x11\n\ttimestamp\x18\x02 \x01(\x03\x12\x11\n\tchangeset\x18\x03 \x01(\x03\x12\x0b\n\x03uid\x18\x04 \x01(\x05\x12\x10\n\x08user_sid\x18\x05 \x01(\r\x12\x0f\n\x07visible\x18\x06 \x01(\x08\"\x8a\x01\n\tDenseInfo\x12\x13\n\x07version\x18\x01 \x03(\x05\x42\x02\x10\x01\x12\x15\n\ttimestamp\x18\x02 \x03(\x12\x42\x02\x10\x01\x12\x15\n\tchangeset\x18\x03 \x03(\x12\x42\x02\x10\x01\x12\x0f\n\x03uid\x18\x04 \x03(\x11\x42\x02\x10\x01\x12\x14\n\x08user_sid\x18\x05 \x03(\x11\x42\x02\x10\x01\x12\x13\n\x07visible\x18\x06 \x03(\x08\x42\x02\x10\x01\"\x17\n\tChangeSet\x12\n\n\x02id\x18\x01 \x02(\x03\"l\n\x04Node\x12\n\n\x02id\x18\x01 \x02(\x12\x12\x10\n\x04keys\x18\x02 \x03(\rB\x02\x10\x01\x12\x10\n\x04vals\x18\x03 \x03(\rB\x02\x10\x01\x12\x1a\n\x04info\x18\x04 \x01(\x0b\x32\x0c.OSMPBF.Info\x12\x0b\n\x03lat\x18\x08 \x02(\x12\x12\x0b\n\x03lon\x18\t \x02(\x12\"{\n\nDenseNodes\x12\x0e\n\x02id\x18\x01 \x03(\x12\x42\x02\x10\x01\x12$\n\tdenseinfo\x18\x05 \x01(\x0b\x32\x11.OSMPBF.DenseInfo\x12\x0f\n\x03lat\x18\x08 \x03(\x12\x42\x02\x10\x01\x12\x0f\n\x03lon\x18\t \x03(\x12\x42\x02\x10\x01\x12\x15\n\tkeys_vals\x18\n \x03(\x05\x42\x02\x10\x01\"\x85\x01\n\x03Way\x12\n\n\x02id\x18\x01 \x02(\x03\x12\x10\n\x04keys\x18\x02 \x03(\rB\x02\x10\x01\x12\x10\n\x04vals\x18\x03 \x03(\rB\x02\x10\x01\x12\x1a\n\x04info\x18\x04 \x01(\x0b\x32\x0c.OSMPBF.Info\x12\x10\n\x04refs\x18\x08 \x03(\x12\x42\x02\x10\x01\x12\x0f\n\x03lat\x18\t \x03(\x12\x42\x02\x10\x01\x12\x0f\n\x03lon\x18\n \x03(\x12\x42\x02\x10\x01\"\xe0\x01\n\x08Relation\x12\n\n\x02id\x18\x01 \x02(\x03\x12\x10\n\x04keys\x18\x02 \x03(\rB\x02\x10\x01\x12\x10\n\x04vals\x18\x03 \x03(\rB\x02\x10\x01\x12\x1a\n\x04info\x18\x04 \x01(\x0b\x32\x0c.OSMPBF.Info\x12\x15\n\troles_sid\x18\x08 \x03(\x05\x42\x02\x10\x01\x12\x12\n\x06memids\x18\t \x03(\x12\x42\x02\x10\x01\x12.\n\x05types\x18\n \x03(\x0e\x32\x1b.OSMPBF.Relation.MemberTypeB\x02\x10\x01\"-\n\nMemberType\x12\x08\n\x04NODE\x10\x00\x12\x07\n\x03WAY\x10\x01\x12\x0c\n\x08RELATION\x10\x02\x42\x0f\n\rcrosby.binary')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'earth_osm.osmpbf.osmformat_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\rcrosby.binary'
  _DENSEINFO.fields_by_name['version']._options = None
  _DENSEINFO.fields_by_name['version']._serialized_options = b'\020\001'
  _DENSEINFO.fields_by_name['timestamp']._options = None
  _DENSEINFO.fields_by_name['timestamp']._serialized_options = b'\020\001'
  _DENSEINFO.fields_by_name['changeset']._options = None
  _DENSEINFO.fields_by_name['changeset']._serialized_options = b'\020\001'
  _DENSEINFO.fields_by_name['uid']._options = None
  _DENSEINFO.fields_by_name['uid']._serialized_options = b'\020\001'
  _DENSEINFO.fields_by_name['user_sid']._options = None
  _DENSEINFO.fields_by_name['user_sid']._serialized_options = b'\020\001'
  _DENSEINFO.fields_by_name['visible']._options = None
  _DENSEINFO.fields_by_name['visible']._serialized_options = b'\020\001'
  _NODE.fields_by_name['keys']._options = None
  _NODE.fields_by_name['keys']._serialized_options = b'\020\001'
  _NODE.fields_by_name['vals']._options = None
  _NODE.fields_by_name['vals']._serialized_options = b'\020\001'
  _DENSENODES.fields_by_name['id']._options = None
  _DENSENODES.fields_by_name['id']._serialized_options = b'\020\001'
  _DENSENODES.fields_by_name['lat']._options = None
  _DENSENODES.fields_by_name['lat']._serialized_options = b'\020\001'
  _DENSENODES.fields_by_name['lon']._options = None
  _DENSENODES.fields_by_name['lon']._serialized_options = b'\020\001'
  _DENSENODES.fields_by_name['keys_vals']._options = None
  _DENSENODES.fields_by_name['keys_vals']._serialized_options = b'\020\001'
  _WAY.fields_by_name['keys']._options = None
  _WAY.fields_by_name['keys']._serialized_options = b'\020\001'
  _WAY.fields_by_name['vals']._options = None
  _WAY.fields_by_name['vals']._serialized_options = b'\020\001'
  _WAY.fields_by_name['refs']._options = None
  _WAY.fields_by_name['refs']._serialized_options = b'\020\001'
  _WAY.fields_by_name['lat']._options = None
  _WAY.fields_by_name['lat']._serialized_options = b'\020\001'
  _WAY.fields_by_name['lon']._options = None
  _WAY.fields_by_name['lon']._serialized_options = b'\020\001'
  _RELATION.fields_by_name['keys']._options = None
  _RELATION.fields_by_name['keys']._serialized_options = b'\020\001'
  _RELATION.fields_by_name['vals']._options = None
  _RELATION.fields_by_name['vals']._serialized_options = b'\020\001'
  _RELATION.fields_by_name['roles_sid']._options = None
  _RELATION.fields_by_name['roles_sid']._serialized_options = b'\020\001'
  _RELATION.fields_by_name['memids']._options = None
  _RELATION.fields_by_name['memids']._serialized_options = b'\020\001'
  _RELATION.fields_by_name['types']._options = None
  _RELATION.fields_by_name['types']._serialized_options = b'\020\001'
  _HEADERBLOCK._serialized_start=45
  _HEADERBLOCK._serialized_end=308
  _HEADERBBOX._serialized_start=310
  _HEADERBBOX._serialized_end=380
  _PRIMITIVEBLOCK._serialized_start=383
  _PRIMITIVEBLOCK._serialized_end=593
  _PRIMITIVEGROUP._serialized_start=596
  _PRIMITIVEGROUP._serialized_end=779
  _STRINGTABLE._serialized_start=781
  _STRINGTABLE._serialized_end=805
  _INFO._serialized_start=807
  _INFO._serialized_end=920
  _DENSEINFO._serialized_start=923
  _DENSEINFO._serialized_end=1061
  _CHANGESET._serialized_start=1063
  _CHANGESET._serialized_end=1086
  _NODE._serialized_start=1088
  _NODE._serialized_end=1196
  _DENSENODES._serialized_start=1198
  _DENSENODES._serialized_end=1321
  _WAY._serialized_start=1324
  _WAY._serialized_end=1457
  _RELATION._serialized_start=1460
  _RELATION._serialized_end=1684
  _RELATION_MEMBERTYPE._serialized_start=1639
  _RELATION_MEMBERTYPE._serialized_end=1684
# @@protoc_insertion_point(module_scope)
